Attribute VB_Name = "modGUID"
Option Explicit

'
' API stuff needed for GenerateGUID.
'
Private Declare Function CoCreateGuid Lib "ole32" (ByRef pguid As GUID) As Long
Private Declare Function StringFromGUID2 Lib "ole32" (ByRef rguid As Any, ByVal lpstrClsId As Long, ByVal cbMax As Long) As Long

Private Type GUID
    Data1 As Long
    Data2 As Integer
    Data3 As Integer
    Data4(0 To 7) As Byte
End Type

'
' API stuff needed for GenerateGUIDEx.
'
Private Declare Function UuidCreate Lib "rpcrt4" (ByRef lpGUID As GUID) As Long
Private Declare Function UuidToString Lib "rpcrt4" Alias "UuidToStringA" (ByRef lpGUID As GUID, ByRef lpGUIDString As Long) As Long
Private Declare Function RpcStringFree Lib "rpcrt4" Alias "RpcStringFreeA" (ByRef lpGUIDString As Long) As Long

Private Const RPC_S_OK As Long = 0&
Private Const RPC_S_UUID_LOCAL_ONLY As Long = 1824&
Private Const RPC_S_UUID_NO_ADDRESS As Long = 1739&

'Private Type GUID
'    Data1 As Long
'    Data2 As Integer
'    Data3 As Integer
'    Data4(0 To 7) As Byte
'End Type

Private Declare Function lstrlen Lib "kernel32" Alias "lstrlenA" (ByVal lpString As Long) As Long
Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (ByRef lpDest As Any, ByRef lpSource As Any, ByVal cBytes As Long)

Private m_blnBrackets As Boolean

'***********************************
' Public properties.
'***********************************

Public Property Let Brackets(ByVal blnNewBrackets As Boolean)
    m_blnBrackets = blnNewBrackets
End Property

Public Property Get Brackets() As Boolean
    Brackets = m_blnBrackets
End Property

'***********************************
' Public methods.
'***********************************

'
' GenerateGUID  -  Return a GUID, Globally Unique IDentifier.
' Return        -  The GUID upon success, Null string if an error occurred.
'
Public Function GenerateGUID() As String
    Dim lngReturn As Long
    Dim udtGUID As GUID
    
    ' Generate the GUID.
    lngReturn = CoCreateGuid(udtGUID)
    
    ' If there was an error exit.
    If lngReturn <> 0 Then
        Exit Function
    End If
    
    Dim strGUID As String
    
    ' Build the human readable GUID string.
    strGUID = Format$(Hex$(udtGUID.Data1), "00000000") & "-" & _
        Format$(Hex$(udtGUID.Data2), "0000") & "-" & _
        Format$(Hex$(udtGUID.Data3), "0000") & "-" & _
        Format$(Hex$(udtGUID.Data4(0)), "00") & _
        Format$(Hex$(udtGUID.Data4(1)), "00") & "-" & _
        Format$(Hex$(udtGUID.Data4(2)), "00") & _
        Format$(Hex$(udtGUID.Data4(3)), "00") & _
        Format$(Hex$(udtGUID.Data4(4)), "00") & _
        Format$(Hex$(udtGUID.Data4(5)), "00") & _
        Format$(Hex$(udtGUID.Data4(6)), "00") & _
        Format$(Hex$(udtGUID.Data4(7)), "00")
    
    If m_blnBrackets Then strGUID = "{" & strGUID & "}"
    
    ' Return the string.
    GenerateGUID = strGUID
End Function

Public Function GenerateGUID2() As String
    Dim uGUID As GUID, strGUID As String, bytGUID() As Byte
    Dim m As Long, n As Long
    
    m = 40
    bytGUID = String$(m, 0)
    
    If CoCreateGuid(uGUID) <> 0 Then
        Exit Function
    End If
    
    ' Convert the structure into a displayable string.
    n = StringFromGUID2(uGUID, VarPtr(bytGUID(0)), m)
    strGUID = bytGUID
    If Asc(Mid$(strGUID, n, 1)) = 0 Then n = n - 1
    
    GenerateGUID2 = Left$(strGUID, n)
    
    ' Die Formatierungsfunktion schliesst den GUID bereits in geschwungene Klammern ein, sind
    ' diese nicht erwnscht, mssen sie entfernt werden.
    If Not m_blnBrackets Then GenerateGUID2 = Mid$(GenerateGUID2, 2, Len(GenerateGUID2) - 2)
End Function

'
' Diese Funktion hat genau die gleichen Resultate wie die vorige, nur dass eine andere Methode
' zum Erstellen des GUID verwendet wird.
'
Public Function GenerateGUIDEx() As String
    Dim guidGUID As GUID, lpStrGUID As Long, lngStrLen As Long, strGUID As String
    Dim strBuffer() As Byte
    
    ' Create the GUID.
    If UuidCreate(guidGUID) <> RPC_S_UUID_NO_ADDRESS Then
        
        ' If the GUID was created, convert it to a string since we can't do much with the struct
        ' that holds it now.
        If UuidToString(guidGUID, lpStrGUID) = RPC_S_OK Then
            
            ' Get the length of the string that was returned.
            lngStrLen = lstrlen(lpStrGUID)
            
            ' Dimention our byte array to the proper size.
            ReDim strBuffer(0 To lngStrLen - 1) As Byte
            
            ' Copy the string to the byte array.
            Call CopyMemory(strBuffer(0), ByVal lpStrGUID, lngStrLen)
            
            ' Release the memory for the string that Windows returned.
            Call RpcStringFree(lpStrGUID)
            
            ' Convert the string to unicode so that we can display it. If you don't need to display
            ' the string, you may want to remove this call since converting to unicode doubles the
            ' amount of the storage space required.
            strGUID = UCase$(StrConv(strBuffer, vbUnicode))
            
            If m_blnBrackets Then strGUID = "{" & strGUID & "}"
            
            ' Show it in the label.
            GenerateGUIDEx = strGUID
        End If
    End If
End Function
